###############################################################################
##  Kroller - v0.84
###############################################################################
##  
##  Code by:
##    Brandon Zehm <caspian at dotconf.net>
##  
##  Icon set arranged by:
##    Adam Duskett
##  
##  Kroller is an extension for SuperKaramba to make a MacOS-X like
##  menu bar.
##  It was originally based on tuxbar, but now only a few lines of
##  code from it remain.
##  
##  This SuperKaramba extension was designed to be flexible.
##  You can add new icons, or change existing ones by simply
##  editing the lines in the "User Configurable Section" in this
##  file.  You can use icons of any size you want.  All you need
##  to do is make sure that all the "small" icons are the same
##  size, and all the "big" icons are the same size, and you set
##  those sizes in the imageSize and imageSizeBig variables.
##
##  Feel free to email me with comments or suggestions!
##
###############################################################################
##
##  To Do List
##  
##  - have it automatically move itself to the bottom-center of your screen
##  - shrink icons slightly or somehting when clicked
##  - a k launch bar
##  - a kontrol center bar?
##  - virtual desktop selector
##  - date and time
##  - smooth scrolling from the top?
##  
###############################################################################

## Import a few standard python modules
import string, re

## This import statement allows access to the karamba functions
import karamba





############################
##  Global Configuration  ##
############################
conf = {
    
    ## Configuration file location, Edit this!    
    "configurationFile"      : "/your/path/to/kroller.conf",
    
    "version"                : "v0.84",
    "debug"                  : 0,                  ## Debug level - overwritten by the config file
    "smallImageSize"         : 0,                  ## Size of small images - overwritten by the config file
    "largeImageSize"         : 0,                  ## Size of large images - overwritten by the config file
    
}


## Define some more global variables
commands         = []
imageFiles       = []
imageFilesBig    = []
images           = []
imagesBig        = []
imageSize        = 0
imageSizeBig     = 0
numIcons         = 0
iconSeparation   = 0
centerSmallIcon  = 0
borderLeft       = 0
sizeDiff         = 0
iconFloor        = 0
widgetWidth      = 0
widgetHeight     = 0
init             = 0
reset            = 0

## barimageFile = "pics/osxbarback.png"
## barimage = 0
## barimageBigFile = "pics/osxbarbackBig.png"
## barimageBig = 0













###############################################################################
##  Function:
##    initWidget(widget)
##  
##  Description:
##    This function is buggy.. I think it gets called too early, because
##    if you try to create an image it crashes karamba.
###############################################################################
def initWidget(widget):
    """Initialize Widget"""    
    
    if (conf['debug'] >= 2):
        print "initWidget() got called"
    










###############################################################################
##  Function: 
##    widgetClicked(widget, x, y, button)
##  
##  This gets called everytime our widget is clicked.
##  Notes:
##    widget = reference to our widget
##    x = x position (relative to our widget)
##    y = y position (relative to our widget)
##    botton = button clicked:
##                    1 = Left Mouse Button
##                    2 = Middle Mouse Button
##                    3 = Right Mouse Button, but this will never happen
##                        because the right mouse button brings up the
##                        Karamba menu.
###############################################################################
def widgetClicked(widget, x, y, button):
    """widgetClicked gets called when our widget is clicked"""
    
    if (conf['debug'] >= 2):
        print "widgetClicked() got called"
    










###############################################################################
##  Function:
##    widgetUpdated(widget)
##  
##  Description:
##    Right now this function is only used for initializing the widget
##
###############################################################################
def widgetUpdated(widget):
    """widgetUpdated gets called at program init and every 4 seconds or so"""    
    global init
    global images
    global imagesBig
    global commands
    global imageFiles
    global imageFilesBig
    global images
    global imagesBig
    global imageSize
    global imageSizeBig
    global numIcons
    global iconSeparation
    global centerSmallIcon
    global borderLeft
    global sizeDiff
    global iconFloor
    global widgetHeight
    global widgetWidth
    
    if (conf['debug'] >= 2):
        print "widgetUpdate got called"
    
    ## Do initialization stuff if we need to
    if init == 0:
        
        ## Read the configuration file
        readConfigurationFile(conf['configurationFile'])
        
        ## <calculations>
        
        ## Sizes of your icons (from config file)
        imageSize    = int(conf['smallImageSize'])
        imageSizeBig = int(conf['largeImageSize'])
        
        conf['debug'] = int(conf['debug'])
        
        ## Calculate some other important stuff
        numIcons = len(imageFiles)
        iconSeparation = (imageSizeBig / 2)                      ## The size of the "cells" where icons go
        centerSmallIcon = ((iconSeparation/2)-(imageSize/2))     ## The number of pixels to center each small icon in it's "cell"
        borderLeft = iconSeparation                              ## How much "white" space on the left
        sizeDiff = imageSizeBig - imageSize                      ## How many pixels are there from the small icon to the big icon
        
        
        ## Calculate widget size
        widgetWidth = (borderLeft + (iconSeparation * (numIcons+1)))
        widgetHeight = imageSizeBig * 1.2
        
        ## Figure out where the bottom of the icons should be (the iconFloor)
        ## In this case I'm just putting them just above the bottom
        iconFloor = ((widgetHeight - imageSize) / 1.2)
        
        if (conf['debug'] >= 2):
            print "numIcons: ", numIcons
            print "imageSize: ", imageSize
            print "imageSizeBig:", imageSizeBig
            print "centerSmallIcon: ", centerSmallIcon
            print "iconSeparation: ", iconSeparation
            print "borderLeft: ", borderLeft
            print "iconFloor: ", iconFloor
            print "widgetWidth: ", widgetWidth
            print "widgetHeight: ", widgetHeight
            
        ## </calculations>
        
        
        ## Set the size of our widget since we're just starting
        karamba.resizeWidget(widget, widgetWidth, widgetHeight)
        
        ## Setup a background image
        ## barimage = karamba.createImage(widget, 60, 62, barimageFile)
        ## barimageBig = karamba.createImage(widget, 6, widgetHeight, barimageBigFile)
        
        for i in range (0,numIcons):
            images.append(karamba.createImage(widget, borderLeft + centerSmallIcon + (i*iconSeparation), iconFloor, imageFiles[i]))
            imagesBig.append(karamba.createImage(widget, borderLeft + (i*iconSeparation), widgetHeight, imageFilesBig[i]))
            karamba.createClickArea(widget, borderLeft + (i*iconSeparation), iconFloor + imageSize - imageSizeBig, iconSeparation, imageSizeBig, commands[i])
        
        karamba.redrawWidget(widget)
        reset = 1
        init = 1










###############################################################################
##  Function:   
##    widgetMouseMoved(widget, x, y, button)
##  
##  This gets called everytime the mouse is moved over our widget
##  Notes
##    widget = reference to our widget
##    x = x position (relative to our widget)
##    y = y position (relative to our widget)
##    botton = button being held:
##                    0 = No Mouse Button
##                    1 = Left Mouse Button
##                    2 = Middle Mouse Button
##                    3 = Right Mouse Button, but this will never happen
##                        because the right mouse button brings up the
##                        Karamba menu.
###############################################################################
def widgetMouseMoved(widget, x, y, button):
    """widgetMouseMoved gets called everytime the mouse gets moved over our widget"""    
    global init
    global reset
    
    if init == 1:
        iconNum = -2
        
        ## If mouse is to far right or left reset everything to normal
        if ((x < (borderLeft-iconSeparation)) or (x >= (borderLeft + ((numIcons+1) * iconSeparation) ))) and reset == 0:
            if (conf['debug'] >= 1):
                print "x is out of bounds.."
            resetWidget(widget)
        
        ## Or if the mouse is too low or too high reset everything to normal
        elif ((y < (iconFloor + imageSize - imageSizeBig)) or (y > (iconFloor + imageSize))) and reset == 0:
            if (conf['debug'] >= 1):
                print "y is out of bounds.."
            resetWidget(widget)
        
        ## If the mouse is in the target area
        elif (x >= (borderLeft-iconSeparation)) and ((y >= (iconFloor + imageSize - imageSizeBig)) and (y <= (iconFloor + imageSize))):
            
            if (conf['debug'] >= 1):
                print "processing.."
            
            ## Which icon is the mouse over?
            iconNum = (x - borderLeft) / iconSeparation   ## borderLeft is initial white-space, and iconSeparation pixels between each icon
            
            ## This returns a number between 0 and iconSeparation depending on how far through the image you are
            offset     = (x - borderLeft) % iconSeparation                  ## 0 - iconSeparation (imageSizeBig/2)
            sizeRatio = ((sizeDiff + 0.0) / imageSizeBig)                   ## Ratio of small icon to big icon
            ## This is the size of the currentIcon, and changes depending on how far through the image you are
            offsetSize = ((offset*2) * sizeRatio)                           ## 0 - sizeDiff
            
            reset = 1
            
            ## Unless the current icon comes up as a non-valid number
            if (-2 < iconNum < numIcons+1):
                
                reset = 0
                
                ## karamba.moveImage(widget, barimage, 60, 160)                
                ## karamba.moveImage(widget, barimageBig, 6, 62)                
                
                
                ################################
                ## Size and position iconNum-1
                ################################
                ## This icon:  x will increase and y will decrease with the increasing offset
                
                ## Unless we're on the first image, resize the icon to our left
                if iconNum > 0:
                    ## Calculate stuff
                    size = imageSize + (sizeDiff/2) - (offsetSize/2)
                    xpos = ( ((iconNum-2) * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) + ((size-imageSize)/(2*sizeRatio)) )   ## (iconSeparation/2)-(size/2) centers the image, then plus ((size-imageSize)/2), half the size we've grown
                    ypos = iconFloor - iconSeparation + offset   ## The iconSeparation variable is here because iconSeparation - offset = 0 at right border
                    
                    ## Place stuff
                    karamba.resizeImage(widget, imagesBig[iconNum-1], size, size)
                    karamba.moveImage(widget, imagesBig[iconNum-1], xpos, ypos)
                    
                    ## Make sure the current small icon is off the screen
                    karamba.moveImage(widget, images[iconNum-1], ((iconNum-1)*iconSeparation)+imageSizeBig, widgetHeight)
                    
                    ## Make sure all the previous icons are setup normally
                    for i in range (0, iconNum - 1):
                        karamba.moveImage(widget, images[i], ((i-1)*(iconSeparation)) + borderLeft + centerSmallIcon, iconFloor)
                        karamba.moveImage(widget, imagesBig[i], ((i-1)*(iconSeparation)) + borderLeft, widgetHeight)
                
                    if (conf['debug'] >= 1):
                        print "iconNum: ", iconNum-1, "\toffset: ", offset, "\toffsetSize: ", offsetSize, "\tsize: ", size, "\txpos: ", xpos, "\typos: ", ypos
                
                ################################
                ## Size and position iconNum
                ################################
                ## This icon:  x will increase and y will decrease with the increasing offset
                ##             until offset > (half of the offset), then
                ##             x will decrease and y will increase with the increasing offset
                
                if ( (iconNum > -1) and (iconNum < numIcons) ):
                    ## If we're on the left side of the icon
                    if offset < (iconSeparation/2):  ## If image is less than half of the offset
                        ## Calculate stuff
                        size = imageSize + (sizeDiff/2) + offsetSize
                        xpos = ( (iconNum * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) + (iconSeparation/2) - offset )      ## Moves to the right as offset increases
                        ypos = iconFloor - iconSeparation - (offset)                                      
                        
                        ## Place stuff
                        karamba.resizeImage(widget, imagesBig[iconNum], size, size)
                        karamba.moveImage(widget, imagesBig[iconNum], xpos, ypos)
                    
                    
                    ## If we're on the right side of the icon
                    else:  ## If image is greater than half of the offset
                        ## Calculate stuff
                        size = imageSize + sizeDiff + (sizeDiff/2) - offsetSize          ## the extra sizeDiff/2 is to make things look right because of the changing size
                        ## First set initial placement                  plus   center it               plus  add half of an iconSegment  and subtract the offset (0-iconSeparation pixels)
                        xpos = ( (iconNum * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) + (iconSeparation/2) - offset )
                        ## 
                        ypos = iconFloor - iconSeparation - iconSeparation + offset
                        
                        ## Place stuff
                        karamba.resizeImage(widget, imagesBig[iconNum], size, size)
                        karamba.moveImage(widget, imagesBig[iconNum], xpos, ypos)
                    
                    ## Move the current small image off the widget
                    karamba.moveImage(widget, images[iconNum], (iconNum*iconSeparation) + borderLeft + centerSmallIcon, widgetHeight)
                    
                    if (conf['debug'] >= 1):
                        print "iconNum: ", iconNum, "\toffset: ", offset, "\toffsetSize: ", offsetSize, "\tsize: ", size, "\txpos: ", xpos, "\typos: ", ypos
                    
                
                
                
                
                ################################
                ## Size and position iconNum+1
                ################################
                ## This icon:  x will decrease and y will increase with the increasing offset
                
                ## Unless we're on the last image, resize the icon to our right
                if (iconNum < numIcons-1):
                    ## Calculate stuff
                    size = imageSize + (offsetSize/2)
                    xpos = ( ((iconNum+2) * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) - ((size-imageSize)/(2*sizeRatio)) )   ## Moves to the left as offset increases
                    ypos = iconFloor - offset
                    
                    ## Place stuff
                    karamba.resizeImage(widget, imagesBig[iconNum+1], size, size)
                    karamba.moveImage(widget, imagesBig[iconNum+1], xpos, ypos)
                    
                    ## Make sure the current small icon is off the screen
                    karamba.moveImage(widget, images[iconNum+1], ((iconNum+1)*iconSeparation)+imageSizeBig, widgetHeight)
                    
                    ## Make sure all the following icons are setup normally
                    for i in range (iconNum+2, numIcons):
                        karamba.moveImage(widget, images[i], ((i)*iconSeparation) + iconSeparation + borderLeft + centerSmallIcon, iconFloor)
                        karamba.moveImage(widget, imagesBig[i], ((i)*iconSeparation) + iconSeparation + borderLeft, widgetHeight)
                        
                    if (conf['debug'] >= 1):
                        print "iconNum: ", iconNum+1, "\toffset: ", offset, "\toffsetSize: ", offsetSize, "\tsize: ", size, "\txpos: ", xpos, "\typos: ", ypos
                if (conf['debug'] >= 1):
                    print
                
                
                ## Update the screen with the changes we've just made
                karamba.redrawWidget(widget)










#############################################################################
##  Function:
##    resetWidget(widget)
##  
##  Description:
##    Resets the widget to it's normal state, i.e. no icons zoomed
##  
#############################################################################
def resetWidget(widget):
    """Resets widget to standby state.. i.e. no icons zoomed."""    
    global reset
    
    for i in range (0,numIcons):
        karamba.moveImage(widget, images[i], (i*iconSeparation) + borderLeft + centerSmallIcon, iconFloor)
        karamba.moveImage(widget, imagesBig[i], (i*iconSeparation) + borderLeft, widgetHeight)
        ## karamba.moveImage(widget, barimage, 60, 62)                
        ## karamba.moveImage(widget, barimageBig, 6, widgetHeight)                
    karamba.redrawWidget(widget)
    reset = 1










###############################################################################
## Function:    
##   readConfigurationFile ( string $filename )
## 
## 
## Description: 
##   Reads the file $filename, and stores information from the file into 
##   the global hash %conf.  This version of the function has been modified 
##   specifically to accomadate the needs of Kroller.
## 
## Example: 
##   readConfigurationFile("/etc/configuration.conf");
##
###############################################################################
def readConfigurationFile(filename):
    """Read a configuration file and store name:value pairs into the global variable conf"""
    global commands
    global imageFiles
    global imageFilesBig
    
    ## Open the configuration file
    try:
        file = open(conf['configurationFile'],"r")
    except IOError:
        print
        print "ERROR While opening the configuration file [" + conf['configurationFile'] + "]"
        print "Try editing kroller.py and changing the path to the configuration file."
        print
    
    ## Read through each line of the file
    while 1:
        line = file.readline()                        ## Read a line from the file
        if len(line) == 0:                            ## Break this while loop if we're at the end of the file
            break
        
        line = re.sub("#.*$", "", line)               ## Remove comments
        line = re.sub("//.*$", "", line)              ## Remove comments
        line = line.strip()                           ## Remove line ending characters, and extra whitespace
        if re.search("\s+=>\s+", line):               ## If it looks like a valid configuration line
            
            ## Debugging
            if (conf['debug'] >= 6):
                print "Line is: ", line
            
            ## Extract the name/value pair
            (name, value) = re.split("\s+=>\s+", line, 1)
            
            ## Add a command
            if (name == "addCommand"):
                commands.append(value)
            
            ## Add a small image
            elif (name == "addSmallImage"):
                imageFiles.append(value)
            
            ## Add a large image
            elif (name == "addLargeImage"):
                imageFilesBig.append(value)
            
            ## Must be a generic configuration parameter, store it in conf
            else:
                if (re.match("^\d+$", value)):
                    conf[name] = int(value)
                else:
                    conf[name] = value
        
    ## Close the coniguration file
    file.close()
    
    
    ## DEBUGGING:
    if conf['debug'] >= 3:
        print "Configuration: "
        for key in conf.keys():
            print key, "\t=>\t", conf[key]










# This will be printed when the widget loads.
print "Loaded my python extension!"

